package cokr.xit.fims.cvlc.dao;

import java.util.List;

import org.egovframe.rte.psl.dataaccess.mapper.Mapper;

import cokr.xit.fims.crdn.CrdnQuery;
import cokr.xit.fims.cvlc.CrdnCvlcpt;
import cokr.xit.foundation.component.AbstractMapper;
import cokr.xit.foundation.data.DataObject;

/**민원 답변 문구 관리 DAO
 * @author leebj
 */
@Mapper("answerWordsMapper")
public interface AnswerWordsMapper extends AbstractMapper {

	/**지정한 조건에 따라 민원 답변 기본문구를 조회하여 반환한다.
	 * @param crdnQuery 답변 기본문구 설정 조회조건
	 * @return 답변문구 정보
	 */
	DataObject selectCvlcptAnswerBasicWordsInfo(CrdnQuery crdnQuery);

	/**지정한 조건에 따라 민원 답변 본문문구를 조회하여 반환한다.
	 * @param crdnQuery 답변 본문문구 설정 조회조건
	 * @return 답변문구 정보
	 */
	DataObject selectCvlcptAnswerWordsInfo(CrdnQuery crdnQuery);

	/**지정한 조건에 따라 부과처리일 경우의 민원 답변 본문문구 목록을 조회한다.
	 * @param crdnQuery 답변 본문 문구 조회조건
	 * @return 답변문구 목록
	 */
	List<DataObject> selectLevyCaseWordsList(CrdnQuery crdnQuery);

	/**지정한 조건에 따라 계도처리일 경우의 민원 답변 본문문구 목록을 조회한다.
	 * @param crdnQuery 답변 본문 문구 조회조건
	 * @return 답변문구 목록
	 */
	List<DataObject> selectWarningCaseWordsList(CrdnQuery crdnQuery);

	/**지정한 조건에 따라 미부과처리일 경우의 민원 답변 본문문구 목록을 조회한다.
	 * @param crdnQuery 답변 본문 문구 조회조건
	 * @return 답변문구 목록
	 */
	List<DataObject> selectNonlevyCaseWordsList(CrdnQuery crdnQuery);

	/** 민원답변 기본문구를 등록한다.
	 * @param crdnCvlcpt 민원답변 기본문구
	 * @return 저장된 정보 수
	 */
	int insertAnswerBasicWordsInfo(CrdnCvlcpt crdnCvlcpt);

	/** 민원답변 기본문구를 변경한다.
	 * @param crdnCvlcpt 민원답변 기본문구
	 * @return 저장된 정보 수
	 */
	int updateAnswerBasicWordsInfo(CrdnCvlcpt crdnCvlcpt);

	/** 민원답변 본문문구를 등록한다.
	 * @param crdnCvlcpt 민원답변 본문문구
	 * @return 저장된 정보 수
	 */
	int insertAnswerBodyWordsInfo(CrdnCvlcpt crdnCvlcpt);

	/** 민원답변 본문문구를 변경한다.
	 * @param crdnCvlcpt 민원답변 본문문구
	 * @return 저장된 정보 수
	 */
	int updateAnswerBodyWordsInfo(CrdnCvlcpt crdnCvlcpt);

	/** 미부과처리건 민원답변 본문문구를 삭제한다.
	 * @param crdnCvlcpt 민원답변 정보
	 * @return 저장된 정보 수
	 */
	int deleteCvlcptAnswerWordsInfo(CrdnCvlcpt crdnCvlcpt);
}
